package MT::Plugin::SKR::WordAlert;

use strict;
use MT;
use MT::Author;
use MT::Entry;
use MT::Template;
use MT::Util qw( is_valid_email );

### Global template name of notification mail.
my $MAIL_TEMPLATE_NAME = 'WordAlert Mail Template';
### Subject of the notification mail
my $MAIL_SUBJECT = '[MovableType/WordAlert] Word Notification Mail';

use vars qw( $MYNAME $VERSION );
$MYNAME = 'WordAlert';
$VERSION = '1.00';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
    name => $MYNAME,
    id => lc $MYNAME,
    key => lc $MYNAME,
    version => $VERSION,
    author_name => 'SKYARC System Co.,Ltd.',
    author_link => 'http://www.skyarc.co.jp/',
    doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/wordalert.html',
    description => <<PERLHEREDOC,
<__trans phrase="Send you a notification email when your favorite words are in posted entries.">
PERLHEREDOC
    l10n_class => $MYNAME. '::L10N',
});
MT->add_plugin ($plugin);

sub instance { $plugin; }

### Registry callbacks and tasks
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        callbacks => {
           'MT::App::CMS::template_source.edit_author' => \&_edit_author_source,
           'MT::App::CMS::template_param.edit_author' => \&_edit_author_param,
           'MT::Author::post_save' => \&_author_post_save,
           'MT::Entry::post_save' => \&_entry_post_save,
        },
        tasks => {
            $MYNAME => {
                name        => $MYNAME,
                frequency   => 60 * 5,# seconds
                code        => \&send_notify_mail,
            },
        },
    });
}



### Edit template source to add the input field for WordAlert
sub _edit_author_source {
    my ($eh, $app, $tmpl) = @_;

    my $old = quotemeta(<<PERLHEREDOC);
<mt:setvarblock name="action_buttons">
PERLHEREDOC
    my $new = &instance->translate(<<PERLHEREDOC);
            <fieldset>
                <h3><__trans phrase="Alert Words List"><a href="javascript:void(0)" onclick="return openManual('author_permissions', 'introduction')" class="help-link">?</a></h3>
    <mtapp:setting
        id="word_alert"
        label="<__trans phrase="Alert Words List">"
        content_class="field-content-text"
        show_label="1">
    <textarea name="alert_word_list" class="full-width" rows="8"><TMPL_VAR NAME=ALERT_WORD_LIST ESCAPE=HTML></textarea>
    </mtapp:setting>
            </fieldset>
PERLHEREDOC
    $$tmpl =~ s/($old)/$new$1/;
}

### Load and Set parameters for user configuration
sub _edit_author_param {
    my ($eh, $app, $param) = @_;
    my $q = $app->{query}
        or return;
    my $author_id = $q->param('id')
        or return;
    my $data = load_plugindata (key_name ($author_id)) || {};

    $param->{alert_word_list} = $data->{alert_word_list} || '';
}

### Save configurations in user configuration screen
sub _author_post_save {
    my ($eh, $obj) = @_;
    my $q = MT->instance->{query}
        or return;
    my $alert_word_list = $q->param('alert_word_list')
        or return;

    my %data = (
        alert_word_list => $alert_word_list,
    ); 
    save_plugindata (key_name ($obj->id), \%data);
}



### Make a mark when entry posts.
use constant KEY_NAME => 'entries';

sub _entry_post_save {
    my ($eh, $obj) = @_;

    my $data = load_plugindata (KEY_NAME) || {};
    $data->{$obj->id} = $obj->status;
    save_plugindata (KEY_NAME, $data);
}



### Periodic runnable procedure
sub send_notify_mail {
    # Load mail template and compile
    my $tmpl = MT::Template->load({
        blog_id => 0, name => $MAIL_TEMPLATE_NAME,
    }) or die "Could not load notification mail template - $MAIL_TEMPLATE_NAME";
    my $ctx = $tmpl->context();
    require MT::Builder;
    my $builder = MT::Builder->new;
    my $tokens = $builder->compile ($ctx, $tmpl->text)
        or die "Could not compile the mail template - $MAIL_TEMPLATE_NAME";

    # Load the updated entries
    my $entries = load_plugindata (KEY_NAME) || {};
    # And soon reset the updated entries
    save_plugindata (KEY_NAME, {});

    # Loop with authors
    my $auth_iter = MT::Author->load_iter({ status => MT::Author::ACTIVE() });
    while (my $author = $auth_iter->()) {
        defined $author->email && $author->email ne '' && is_valid_email ($author->email)
            or next;
        my $settings = load_plugindata (key_name ($author->id))
            or next;
        my $alert_word_list = $settings->{alert_word_list}
            or next;
        # Loop with updated entries
        my @entries;
        foreach my $eid (keys %$entries) {
            # Only for the released entry
            if ($entries->{$eid} != MT::Entry::RELEASE()) {
                delete $entries->{$eid};# for next turns
                next;
            }
            # Load an instance
            my $entry = MT::Entry->load({ id => $eid })
                or next;
            my $whole_text =
                ($entry->title || ''). "\n".
                ($entry->text || ''). "\n".
                ($entry->text_more || ''). "\n".
                ($entry->excerpt || ''). "\n".
                ($entry->keywords || ''). "\n";

            # Search with word list
            my $found1 = 0;
            foreach (split /[\r\n]+/, $alert_word_list) {
                s/^\s+|\s+$//g;
                my @words = split /\s+/;
                my $found2 = 1;
                foreach (@words) {
                    $found2 &&= $whole_text =~ /$_/;
                }
                $found1 ||= $found2;
                last if $found1;
            }
            push @entries, $entry if $found1;
        }
        # found no entry with your favorite words.
        next unless scalar @entries;

        # Build mail body
        $ctx->stash ('entries', \@entries);
        my $mail_body = $builder->build ($ctx, $tokens)
            or die $builder->errstr();

        # And send it to the author.
        require MT::Mail;
        my $cfg = MT->instance->config;
        my $from_addr;
        my $reply_to;
        if ($cfg->EmailReplyTo) {
            $reply_to = $cfg->EmailAddressMain;
        }
        else {
            $from_addr = $cfg->EmailAddressMain;
        }
        $from_addr = undef if $from_addr && !is_valid_email ($from_addr);
        $reply_to  = undef if $reply_to  && !is_valid_email ($reply_to);

        my $charset = $cfg->MailEncoding || $cfg->PublishCharset;
        my %head = (
            id => $MYNAME. '_notification_email',
            To => $author->email,
            $from_addr ? ( From => $from_addr ) : (),
            $reply_to  ? ( 'Reply-To' => $reply_to ) : (),
            Subject => &instance->translate ($MAIL_SUBJECT),
            'Content-Type' => qq(text/plain; charset="$charset"),
        );
        MT::Mail->send( \%head, $mail_body )
            or die MT::Mail->errstr();
    }
}



########################################################################
sub key_name { 'author_id::'. $_[0]; }

use MT::PluginData;

sub save_plugindata {
    my ($key, $data_ref) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key });
    if (!$pd) {
        $pd = MT::PluginData->new;
        $pd->plugin( &instance->id );
        $pd->key( $key );
    }
    $pd->data( $data_ref );
    $pd->save;
}

sub load_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key })
        or return undef;
    $pd->data;
}

sub remove_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key })
        or return undef;# there is no entry all along
    $pd->remove;
}

1;
__END__
